package simple;

import java.io.*;

/**
 *	Formal Semantics of Programming Languages
 *	SS 99
 *	1. Exercise
 *
 *	Class represents syntax domain ArrayAssign-Command.
 *	C ::= I[E1] := E2
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@version 1.0 (JDK 1.1.7)
 */
public class CommandArrayAssign extends Command
{
	Identifier ident = null;
	Expression exp1 = null;
	Expression exp2 = null;
	
	public String toString()
	{
		return "(CommandArrayAssign: " + ident.toString() + ", " + exp1.toString() + ", " + exp2.toString() + ")";
	}
	
	public void intendPrint(PrintStream out, String intend)
	{
		out.print("\n" + intend);
		ident.intendPrint(out, intend);
		out.print("[");
		exp1.intendPrint(out, intend);
		out.print("] := ");
		exp2.intendPrint(out, intend);
	}
	
	Poststore execute(Environment e, Store s)
	{
		DenotableValue dval = e.access(ident.execute());
		Location l = null;
		Errvalue err = null;
		Array arr = null;
		ConstantValue cval = null;
		
		if ((l = dval.isLocation()) != null)
		{
			return Poststore.inErr(s);
		}
		else
		if ((arr = dval.isArray()) != null)
		{
			ExpressibleValue xidx = exp1.execute(e, s);
			Num idx = null;
			
			if ((idx = xidx.isNum()) != null)
			{
				ReferenceableValue rval = arr.access(idx);
				Location ref = null;
				
				if ((ref = rval.isLocation()) != null)
				{
					ExpressibleValue xval = exp2.execute(e, s);
					Num n = null;
					
					if ((n = xval.isNum()) != null)
					{
						return s.update(ref, new StorableValue(n)); // conversion needed
					}
					else
					if ((err = xval.isErrvalue()) != null)
					{
						return Poststore.inErr(s);
					}
				}
				else
				if ((err = rval.isErrvalue()) != null)
				{
					return Poststore.inErr(s);
				}
			}
			else
			if ((err = xidx.isErrvalue()) != null)
			{
				return Poststore.inErr(s);
			}
		}
		else
		if ((cval = dval.isConstantValue()) != null)
		{
			return Poststore.inErr(s);
		}
		if ((err = dval.isErrvalue()) != null)
		{
			return Poststore.inErr(s);
		}
		return null;
	}

	public CommandArrayAssign(Identifier ident, Expression exp1, Expression exp2)
	{
		this.ident = ident;
		this.exp1 = exp1;
		this.exp2 = exp2;
	}
}