package simple;

import java.io.*;

/**
 *	Formal Semantics of Programming Languages
 *	SS 99
 *	1. Exercise
 *
 *	Class represents domain While-Command.
 *	C ::= while B do K
 *
 *	While command is executed iterative (not recursive)
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@version 1.0 (JDK 1.1.7)
 */
public class CommandWhile extends Command
{
	BooleanExpr bexp = null;
	Block block = null;

	public String toString()
	{
		return "(CommandWhile: " + bexp.toString() + ", " + block.toString() + ")";
	}
	
	public void intendPrint(PrintStream out, String intend)
	{
		out.print("\n" + intend + "while ");
		bexp.intendPrint(out, intend);
		out.print(" do");
		block.intendPrint(out, intend);
	}
	
	Poststore execute(Environment e, Store s)
	{
		BooleanExprValue bval = null;
		Tr tr = null;
		Errvalue err = null;
		Poststore p = null;
		
		while (true)
		{
			bval = bexp.execute(e, s);
			if ((tr = bval.isTr()) != null)
			{
				if (tr.isTrue() != null)
				{
					p = block.execute(e, s);
					if ((s = p.isErr()) != null)
					{
						return Poststore.inErr(s);
					}
					else
					if ((s = p.isOk()) != null)
					{
						// iterate again
					}
					else
					{
						return null;
					}
				}
				else
				if (tr.isFalse() != null)
				{
					return Poststore.inOk(s);
				}
				else
				{
					return null;
				}
			}
			else
			if ((err = bval.isErrvalue()) != null)
			{
				return Poststore.inErr(s);
			}
		}
		/* Recursive: */
		/*
		Store s1 = null;
		bval = bexp.execute(e, s);
		if ((tr = bval.isTr()) != null)
		{
			if (tr.isTrue() != null)
			{
				p = block.execute(e, s);
				if ((s1 = p.isErr()) != null)
				{
					return Poststore.inErr(s1);
				}
				else
				if ((s1 = p.isOk()) != null)
				{
					return this.execute(e, s1);
				}
			}
			else
			if (tr.isFalse() != null)
			{
				return Poststore.inOk(s);
			}
		}
		else
		if ((err = bval.isErrvalue()) != null)
		{
			return Poststore.inErr(s);
		}
		return null;
		*/
	}
	
	public CommandWhile(BooleanExpr bexp, Block block)
	{
		this.bexp = bexp;
		this.block = block;
	}
}