package simple;

import java.io.*;

/**
 *	Formal Semantics of Programming Languages
 *	SS 99
 *	1. Exercise
 *
 *	Class represents domain Array-identifier-expression.
 *	E ::= I[E]
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@version 1.0 (JDK 1.1.7)
 */
public class ExpressionArrayIdentifier extends Expression
{
	Identifier ident = null;
	Expression exp = null;

	public String toString()
	{
		return "(ExpressionArrayIdentifier: " + ident.toString() + ", " + exp.toString() + ")";
	}
	
	public void intendPrint(PrintStream out, String intend)
	{
		ident.intendPrint(out, intend);
		out.print("[");
		exp.intendPrint(out, intend);
		out.print("]");
	}
	
	ExpressibleValue execute(Environment e, Store s)
	{
		DenotableValue dval = e.access(ident.execute());
		Location l = null;
		ConstantValue cval = null;
		Array arr = null;
		Errvalue err = null;
		
		if ((l = dval.isLocation()) != null)
		{
			return ExpressibleValue.inErrvalue();
		}
		else
		if ((cval = dval.isConstantValue()) != null)
		{
			return ExpressibleValue.inErrvalue();
		}
		else
		if ((arr = dval.isArray()) != null)
		{
			ExpressibleValue xval = exp.execute(e, s);
			Num n = null;
			
			if ((n = xval.isNum()) != null)
			{
				ReferenceableValue rval = arr.access(n);
				Location ref = null;
				
				if ((ref = rval.isLocation()) != null)
				{
					StoreaccessValue aval = s.access(ref);
					StorableValue sval = null;
					
					if ((sval = aval.isStorableValue()) != null)
					{
						return ExpressibleValue.inNum(new Num(sval)); // conversion needed
					}
					else
					if ((err = aval.isErrvalue()) != null)
					{
						return ExpressibleValue.inErrvalue();
					}
				}
				else
				if ((err = rval.isErrvalue()) != null)
				{
					return ExpressibleValue.inErrvalue();
				}
			}
			else
			if ((err = xval.isErrvalue()) != null)
			{
				return ExpressibleValue.inErrvalue();
			}
		}
		else
		if ((err = dval.isErrvalue()) != null)
		{
			return ExpressibleValue.inErrvalue();
		}
		return null;
	}
	
	public ExpressionArrayIdentifier(Identifier ident, Expression exp)
	{
		this.ident = ident;
		this.exp = exp;
	}
}